;; Generated package description from tam.el  -*- no-byte-compile: t -*-
(define-package "tam" "0.1" "Manage use of slots in a fixed size table" '((queue "0.2") (emacs "24.3")) :commit "c254ec9f646ef5527eb1f834a90e5897caa977cf" :authors '(("Onnie Lynn Winebarger" . "owinebar@gmail.com")) :maintainer '("Onnie Lynn Winebarger" . "owinebar@gmail.com") :keywords '("lisp" "tools") :url "https://github.com/owinebar/emacs-table-allocation-manager")
